#ifndef _praat_Tiers_h_
#define _praat_Tiers_h_
/* praat_Tiers.h
 *
 * Copyright (C) 2016,2017 Paul Boersma
 *
 * This code is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or (at
 * your option) any later version.
 *
 * This code is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this work. If not, see <http://www.gnu.org/licenses/>.
 */

/*
	User interface functions for PointProcess and subclasses of RealTier.
*/

#include "AmplitudeTierEditor.h"
#include "DurationTierEditor.h"
#include "FormantGridEditor.h"
#include "FormantTier.h"
#include "IntensityTierEditor.h"
#include "PitchTierEditor.h"
#include "PointEditor.h"
#include "praat_TimeTier.h"

void praat_Tiers_init ();

/* End of file praat_Tiers.h */
#endif
