/* manual_references.cpp
 *
 * Copyright (C) 1992-2008,2010,2011,2014-2017,2019,2023,2025 Paul Boersma
 *
 * This code is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or (at
 * your option) any later version.
 *
 * This code is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this work. If not, see <http://www.gnu.org/licenses/>.
 */

#include "ManPagesM.h"

void manual_references_init (ManPages me);
void manual_references_init (ManPages me) {

MAN_PAGES_BEGIN R"~~~(
################################################################################
"Archangeli & Pulleyblank (1994)"
© Paul Boersma 1997-10-21

Diana Archangeli & Douglas Pulleyblank (1994): %%Grounded phonology%.
Cambridge, MA: MIT Press.

################################################################################
"Boersma (1993)"
© Paul Boersma 2019-03-31, 2023

Paul Boersma (1993): “Accurate short-term analysis of the fundamental frequency
and the harmonics-to-noise ratio of a sampled sound.”
%%Proceedings of the Institute of Phonetic Sciences% #17: 97–110. University of Amsterdam.

Available on `https://www.fon.hum.uva.nl/paul/`.

################################################################################
"Boersma (1997)"
© Paul Boersma 1998-12-19, 2023, 2025

Paul Boersma (1997): “How we learn variation, optionality, and probability.”
%%Proceedings of the Institute of Phonetic Sciences% #21: 43–58. University of Amsterdam.

Available on `https://www.fon.hum.uva.nl/paul/`. Equals chapter 15 of @@Boersma (1998)@.

A less correct version (demoting and promoting a single pair of constraints, instead of them all)
is available as Rutgers Optimality Archive #221, `https://roa.rutgers.edu`.

################################################################################
"Boersma (1998)"
© Paul Boersma 2004-10-20, 2023

Paul Boersma (1998): %%Functional Phonology% [%%LOT International Series% #11].
The Hague: Holland Academic Graphics. Pages i–ix, 1\--493. [Doctoral thesis, University of Amsterdam]

This book can be downloaded as a PDF file from `https://www.fon.hum.uva.nl/paul/`,
where you can also find many Praat scripts for the simulations and pictures in this book.
A paperback version is also available from the author (`paul.boersma@uva.nl`).

################################################################################
"Boersma (2000)"
© Paul Boersma 2000-10-27

Paul Boersma (2000): “Learning a grammar in Functional Phonology.”
In Joost Dekkers, Frank van der Leeuw, & Jeroen van de Weijer (eds.):
%%Phonology, Syntax, and Acquisition in Optimality Theory%. Oxford University Press.

An extended version is chapter 14 of @@Boersma (1998)@.

################################################################################
"Boersma (2009a)"
© Paul Boersma 2019-03-31, 2025

Paul Boersma (2009): “Should jitter be measured by peak picking or by waveform matching?”
%%Folia Phoniatrica et Logopaedica% #61: 305–308.

Available on `https://www.fon.hum.uva.nl/paul/`.

################################################################################
"Boersma (2009b)"
© Paul Boersma 2019-03-31, 2023

Paul Boersma (2009b): “Some correct error-driven versions of the Constraint Demotion algorithm.”
%%Linguistic Inquiry% #40: 667–686.

Available on `https://www.fon.hum.uva.nl/paul/`.

################################################################################
"Boersma & Escudero (2008)"
© Paul Boersma 2019-03-31, 2023

Paul Boersma & Paola Escudero (2008): “Learning to perceive a smaller L2 vowel inventory:
an Optimality Theory account.” In: Peter Avery, Elan Dresher & Keren Rice (eds.),
%%Contrast in phonology: theory, perception, acquisition%. Berlin: Mouton De Gruyter. 271–301.

Available on `https://www.fon.hum.uva.nl/paul/`.

################################################################################
"Boersma & Hayes (2001)"
© Paul Boersma 2019-03-31, 2023

Paul Boersma & Bruce Hayes (2001): “Empirical tests of the Gradual Learning Algorithm.”
%%Linguistic Inquiry% #32: 45–86.

Available on `https://www.fon.hum.uva.nl/paul/`.

################################################################################
"Boersma & Kovacic (2006)"
© Paul Boersma 2019-03-31, 2023

Paul Boersma & Gordana Kovacic (2006):
“Spectral characteristics of three styles of Croatian folk singing.”
%%Journal of the Acoustical Society of America% #119: 1805–1816.

Available on `https://www.fon.hum.uva.nl/paul/`.

################################################################################
"Boersma & Pater (2016)"
© Paul Boersma 2019-03-31, 2023

Paul Boersma & Joe Pater (2016):
“Convergence properties of a gradual learning algorithm for Harmonic Grammar.”
In John J. McCarthy & Joe Pater (eds.),
%%Harmonic Serialism and Harmonic Grammar%. Sheffield: Equinox. 389–434.

Available on `https://www.fon.hum.uva.nl/paul/`.

################################################################################
"Childers (1978)"
© Paul Boersma 2003-05-15

%%Modern spectrum analysis%. IEEE Press.

The Burg algorithm for linear prediction coefficients is described on pages 252\--255.

################################################################################
"Deliyski (1993)"
© Paul Boersma 2003-03-12

Dimitar D. Deliyski: “Acoustic model and evaluation of pathological voice production.”
%%Proceedings Eurospeech ’93%, Vol. 3, 1969\--1972.

################################################################################
"Escudero & Boersma (2004)"
© Paul Boersma 2005-04-27

Paola Escudero & Paul Boersma (2004): “Bridging the gap between L2 speech perception
and phonological theory.” %%Studies in Second Language Acquisition% #26: 551\--585.

################################################################################
"Escudero, Boersma, Rauber & Bion (2009)"
© Paul Boersma 2019-03-31, 2025

Paola Escudero, Paul Boersma, Andréia Schurt Rauber & Ricardo A.H. Bion (2009):
“A cross-dialect acoustic description of vowels: Brazilian and European Portuguese.”
%%Journal of the Acoustical Society of America% #126: 1379\--1393.

Available on `https://www.fon.hum.uva.nl/paul/`.

################################################################################
"Fant (1960)"
© Paul Boersma 1998-02-01

Gunnar Fant (1960): %%Acoustic theory of speech production.% Mouton, The Hague.

################################################################################
"Fischer (2005)"
© Paul Boersma 2010-03-31, 2025

Markus Fischer (2005): “A Robbins-Monro type learning algorithm for an entropy maximizing version
of stochastic Optimality Theory.” Master’s thesis, Humboldt University, Berlin.
[%%Rutgers Optimality Archive% #767, `https://roa.rutgers.edu`]

################################################################################
"Flanagan & Landgraf (1968)"
© Paul Boersma 1998-02-01

James L. Flanagan & L.L. Landgraf (1968): “Self-oscillating source for vocal-tract synthesizers.”
%%IEEE Transactions on Audio and Electroacoustics% ##AU-16#: 57–64.
Reprinted in: James L. Flanagan & Lawrence R. Rabiner (eds.) (1973):
%%Speech synthesis%, Dowden, Hutchinson & Ross, Stroudsburg.

The authors show that if we model each vocal cord as a single mass-spring system,
the vocal cords will vibrate passively as a result of the interaction with a glottal airflow.

################################################################################
"Hayes & MacEachern (1998)"
© Paul Boersma 1998-12-19

Bruce P. Hayes & Margaret MacEachern (1998): “Quatrain form in English folk verse.”
%Language #74: 473\--507.

################################################################################
"Ishizaka & Flanagan (1972)"
© Paul Boersma 1998-02-01

Kenzo Ishizaka & James L. Flanagan (1972): “Synthesis of voiced sounds from a two-mass model
of the vocal cords.” %%Bell System Technical Journal% #51: 1233\--1268.
Reprinted in: James L. Flanagan & Lawrence R. Rabiner (eds.) (1973):
%%Speech Synthesis%, Dowden, Hutchinson & Ross, Stroudsburg.

The authors show that if we model each vocal cord as two coupled mass-spring systems,
the passive vibration that results from the interaction with a glottal airflow
will show more realistic behaviour than with the one-mass model of @@Flanagan & Landgraf (1968)@,
at least for a male speaker.

################################################################################
"Jäger (2003)"
© Paul Boersma 2007-04-23

Gerhard Jäger (2003): “Maximum Entropy Models and Stochastic Optimality Theory.”
To appear in Jane Grimshaw, Joan Maling, Chris Manning, Jane Simpson, and Annie Zaenen (eds.):
%%Architectures, rules, and preferences: A Festschrift for Joan Bresnan%,
CSLI Publications, Stanford.

################################################################################
"Jesteadt, Wier & Green (1977)"
© Paul Boersma 2002-12-15

W. Jesteadt, G.C. Wier, & D.M. Green (1977): “Intensity discrimination as a function
of frequency and sensation level.” %%Journal of the Acoustical Society of America% #61: 169\--177.

################################################################################
"Klatt & Klatt (1990)"
© Paul Boersma 2005-07-12

D.H. Klatt & L.C. Klatt (1990): “Analysis, synthesis and perception of voice quality
variations among male and female talkers.”
%%Journal of the Acoustical Society of America% #87: 820\--856.

################################################################################
"Ladefoged (2001)"
© Paul Boersma 2014-03-25

Peter Ladefoged (2001). %%Vowels and consonants%: %%an introduction to the sounds of languages%.
Oxford: Blackwell. [second edition: 2005; third edition with Sandra Disner: 2012]

A very readable introduction to phonetics, mainly acoustic and articulatory.
Has lots of spectrograms of the sounds of the world’s languages. Comes with a CD that has all those
sounds and includes training material for transcription (from another book).

For a more encyclopaedic treatment of the sounds of the world’s languages,
see @@Ladefoged & Maddieson (1996)@ instead.

################################################################################
"Ladefoged & Maddieson (1996)"
© Paul Boersma 2003-03-16

Peter Ladefoged & Ian Maddieson (1996). %%The sounds of the world’s languages%.
Oxford: Blackwell.

An extensive reference source for the articulation and acoustics of “all” vowels and consonants
that occur in the world’s languages. If you don’t find the answer in this book, you will find the
answer in the articles referred to in this book.
The book uses lots of spectrograms, palatograms, and other techniques.

The book is not an introductory text. For that, see @@Ladefoged (2001)@ instead.

################################################################################
"McCarthy & Prince (1995)"
© Paul Boersma 2002-11-05, 2025

John J. McCarthy & Alan Prince (1995): “Faithfulness and reduplicative identity.”
In Jill Beckman, Laura Walsh Dickey & Suzanne Urbanczyk (eds.), %%Papers in Optimality Theory%.
%%University of Massachusetts Occasional Papers% #18. Amherst, Mass.: Graduate Linguistic Student Association.
pp. 249\--384. [Rutgers Optimality Archive #60, `https://roa.rutgers.edu`]

################################################################################
"Moulines & Charpentier (1990)"
© Paul Boersma 2007-07-22

Eric Moulines & Francis Charpentier (1990): “Pitch-synchronous waveform processing techniques for text-to-speech synthesis using diphones.”
%%Speech Communication% #9: 453\--467.

################################################################################
"Nagarajan, Wang, Merzenich, Schreiner, Johnston, Jenkins, Miller & Tallal (1998)"
© Paul Boersma 2010-10-26

Srikantan S. Nagarajan, Xiaoqin Wang, Michael M. Merzenich, Christoph E. Schreiner, Paul Johnston, William M. Jenkins,
Steven Miller & Paula Tallal (1998): “Speech modifications algorithms used for training language learning-impaired children.”
%%IEEE Transactions on Rehabilitation Engineering% ##6(3)#: 257\--268.

################################################################################
"Pater (2008)"
© Paul Boersma 2010-03-31

Joe Pater (2008): “Gradual learning and convergence.” %%Linguistic Inquiry% #39: 334\--345.

################################################################################
"Pater, Potts & Bhatt (2007)"
© Paul Boersma 2008-03-31

Joe Pater, Christopher Potts & Rajesh Bhatt (2007): “Harmonic Grammar with linear programming.”
Manuscript, University of Massachusetts, Amherst. %%Rutgers Optimality Archive% #872.

################################################################################
"Patterson & Wightman (1976)"
© Paul Boersma 2002-12-15

R. Patterson & F. Wightman (1976): “Residue pitch as a function of component spacing.”
%%Journal of the Acoustical Society of America% #59: 1450\--1459.

################################################################################
"Plomp (1967)"
© Paul Boersma 2002-12-15

Reinier Plomp (1967): “Frequencies dominant in the perception of pitch of complex sounds.”
%%Journal of the Acoustical Society of America% #42: 191\--198.

################################################################################
"Prince & Smolensky (1993)"
© Paul Boersma 1997-10-21

Alan Prince & Paul Smolensky (1993): %%Optimality Theory: Constraint Interaction in Generative Grammar.%
Rutgers University Center for Cognitive Science Technical Report #2.

################################################################################
"Rosenberg (1971)"
© Paul Boersma 2005-07-12

A. Rosenberg (1971): “Effect of glottal pulse shape on the quality of natural vowels.”
%%Journal of the Acoustical Society of America% #49: 583\--590.

################################################################################
"Rosenblatt (1962)"
© Paul Boersma 2007-04-23

Frank Rosenblatt (1962):
%%Principles of neurodynamics; perceptrons and the theory of brain mechanisms%. Washington: Spartan Books.

################################################################################
"Smolensky (1986)"
© Paul Boersma 2010-03-31

Paul Smolensky (1986): “Information processing in dynamical systems: foundations of Harmony Theory.”
In @@Rumelhart & McClelland (1986)@, pp. 194\--281.

################################################################################
"Rumelhart & McClelland (1986)"
© Paul Boersma 2010-03-31

David Rumelhart, James L. McClelland, and the PDP Research Group (eds., 1986):
%%Parallel Distributed Processing: explorations in the microstructure of cognition%. MIT Press, Cambridge.

################################################################################
"Smolensky & Legendre (2006)"
© Paul Boersma 2010-03-31

Paul Smolensky & G\e'raldine Legendre (2006): %%The harmonic mind.%
MIT Press.

These two books contain all material on connectionism by Smolensky and colleagues from the late eighties on,
with extensive editing to make it into a coherent work, plus much new material.

This book can be seen in many respects as the predecessor of @@Prince & Smolensky (1993)@.

If you want to read a single chapter, I recommend @@Soderstrom, Mathis & Smolensky (2006)@.

################################################################################
"Soderstrom, Mathis & Smolensky (2006)"
© Paul Boersma 2007-04-23

Melanie Soderstrom, Donald Mathis & Paul Smolensky (2006):
“Abstract genomic encoding of Universal Grammar in Optimality Theory.”
In @@Smolensky & Legendre (2006)@, pp. 403\--471.

################################################################################
"Tesar & Smolensky (1998)"
© Paul Boersma 2010-03-31, 2025

Bruce Tesar & Paul Smolensky (1998): “Learnability in Optimality Theory.”
%%Linguistic Inquiry% #29: 229\--268.

The first version of the constraint-demotion algorithm appeared in:

Bruce Tesar & Paul Smolensky (1993): “The learnability of Optimality Theory:
an algorithm and some basic complexity results.”
Manuscript Department of Computer Science & Institute of Cognitive Science, University of Colorado at Boulder.
Available as Rutgers Optimality Archive #2, `https://roa.rutgers.edu`.

The Error-Driven Constraint Demotion algorithm (EDCD) can be found in:

Bruce Tesar & Paul Smolensky (1996): “Learnability in Optimality Theory (long version).”
Technical Report ##96-3#, Department of Cognitive Science, Johns Hopkins University, Baltimore.
Available as Rutgers Optimality Archive #156, `https://roa.rutgers.edu`.

A corrected version of EDCD appeared in @@Boersma (2009b)@.

################################################################################
"Willems (1986)"
© Paul Boersma 2002-12-15

Lei Willems (1986): “Robust formant analysis.” %%IPO report% #529: 1\--25.
Eindhoven: Institute for Perception Research.

################################################################################
)~~~" MAN_PAGES_END



}

/* End of file manual_references.cpp */
