/* KlattGrid_PitchTierArea_prefs.h
 *
 * Copyright (C) 2009-2011 David Weenink, 2017,2021,2022 Paul Boersma
 *
 * This code is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or (at
 * your option) any later version.
 *
 * This code is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this work. If not, see <http://www.gnu.org/licenses/>.
 */

Prefs_begin (KlattGrid_PitchTierArea)

	InstancePrefs_overrideDouble (KlattGrid_PitchTierArea, dataFreeMinimum, 1, U"50.0")
	InstancePrefs_overrideDouble (KlattGrid_PitchTierArea, dataFreeMaximum, 1, U"600.0")

Prefs_end (KlattGrid_PitchTierArea)

/* End of file KlattGrid_PitchTierArea_prefs.h */
